/* Construct a linear map from an arbitrary triangle to our standard one, i.e., [-1/3 -1/3 2/3;-1/sqrt(3) 1/sqrt(3) 0].
 *
 * Input:
 * 	C : Triangle corners as a 2x3 matrix.
 * Output:
 *  map : The linear map.
 *  	t=map.T*x+map.c, for t,x\in\mathbb{R}^2, t=arbitrary triangle coords, x=standard triangle coords
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include <boost/numeric/ublas/matrix_proxy.hpp>

class Map ArbitraryTriangleToStandardTriangle(DMatrix C)
{
  Map map;

  map.c(0)=(C(0,0)+C(0,1)+C(0,2))/3.;
  map.c(1)=(C(1,0)+C(1,1)+C(1,2))/3.;
  column(map.T,0)=(column(C,2)-map.c)*1.5;
  column(map.T,1)=(column(C,1)-column(C,0))*0.5*sqrt(3.);
  return map;
}
