/* Construct the Jacobian of a mapping from a nonlinear triangle to the equilateral triangle.
 *
 * Input:
 *	phi (N,N,2) = mapped coordinates of x, i.e., y=phi(x)
 *
 * Output:
 *	J (2,2,N,N) = Jacobian at every point in the domain
 *
 * Brett Ryland, August 2009
 */

#include "Chebyshev.hpp"

JArray JacobianMap(DArray phi)
{
  int N;
  DArray gradphi1,gradphi2;
  JArray J;

  N=phi(0).size1();
  gradphi1=Gradient(phi(0));
  gradphi2=Gradient(phi(1));
  J.resize(N,N);
  for( int i=0; i<N; ++i )
	for( int j=0; j<N; ++j ) {
	  J(i,j).resize(2,2);
	  J(i,j)(0,0)=gradphi1(0)(i,j);
	  J(i,j)(0,1)=gradphi1(1)(i,j);
	  J(i,j)(1,0)=gradphi2(0)(i,j);
	  J(i,j)(1,1)=gradphi2(1)(i,j);
	}
  return J;
}
