/* Evaluate the Chebyshev polynomials in the dual fundamental domain
 * of the A_2 root system at the specified points.
 *
 * Input:
 *	z : The evaluation points as a complex vector, i.e., real(z)=x_1, imag(z)=x_2.
 *	N : Downscaling factor.
 *
 * Output:
 *	The evaluation of all the T_k, where k is in the dual fundamental domain.
 *	The columns of T correspond to the columns of dualfudo(N).
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include <boost/numeric/ublas/matrix_proxy.hpp>

CMatrix T_A2(CVector z, int N)
{
  using namespace boost::numeric::ublas;
  unsigned int j,Nfudo,Nz;
  IMatrix fudo;
  CVector zc;
  CMatrix T;

  fudo=dualfudo(N);
  Nfudo = fudo.size2();
  Nz = z.size();
  zc = boost::numeric::ublas::conj(z);
  T = scalar_matrix<std::complex<double> > (Nz,Nfudo,std::complex<double>(0.,0.));
  column(T,0) = scalar_vector<std::complex<double> > (Nz,std::complex<double>(1.,0.));
  if( N>=2 )
	column(T,1) = z;
  if( N>=4 )
	column(T,2) = 3.*element_prod(z,z)-2.*zc;
  for( j=(N==3?2:3); j<Nfudo; ++j) {
	if( fudo(1,j)==0 )
	  column(T,j) = 3.*element_prod(z,column(T,j-1))-3.*element_prod(zc,column(T,j-2))+column(T,j-3);
	else
	  column(T,j) = (3.*element_prod(column(T,fudo(0,j)),boost::numeric::ublas::conj(column(T,fudo(1,j))))-column(T,fudo(0,j)-fudo(1,j)))*0.5;
  }
  return T;
}
