/* Construct the primal fundamental domain for the A_2 root system.
 * Input:
 * 	N : Downscaling factor.
 * Output:
 * 	The primal fundamental domain for the A_2 root system (optionally sorted for use with Gradient()).
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"

IMatrix primalfudo(int N)
{
  int ndx=0;
  IMatrix fudo;
  fudo.resize(2,N*N); // a bit big but we'll truncate later
  for( int k1=0; k1<=2*N/3; ++k1)
	for( int k2=(int)ceil(k1/2.); k2<=2*k1; ++k2 )
	  if( k1+k2<=N ) {
		fudo(0,ndx) = k1;
		fudo(1,ndx) = k2;
		++ndx;
	  }
  fudo.resize(2,ndx,true);
  return fudo;
}
