#ifndef __Chebyshev
#define __Chebyshev
// mex.h must be included before Chebyshev.hpp if the array_conv() functions are required

#include <complex>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>

// some typedefs of vectors, matrices and arrays. Matrices are row_major by default
typedef boost::numeric::ublas::matrix<double > DMatrix;
typedef boost::numeric::ublas::vector<double > DVector;
typedef boost::numeric::ublas::matrix<int > IMatrix;
typedef boost::numeric::ublas::vector<int > IVector;
typedef boost::numeric::ublas::matrix<std::complex<double> > CMatrix;
typedef boost::numeric::ublas::vector<std::complex<double> > CVector;
typedef boost::numeric::ublas::vector<DMatrix > DArray;
typedef boost::numeric::ublas::vector<IMatrix > IArray;
typedef boost::numeric::ublas::matrix<DMatrix > JArray;

// some missing operators, not sure why these don't just work from the templates...
template<class T>
DArray operator* (const DArray A, const T b)
{
  DArray result=A;
  for( unsigned int i=0; i<A.size(); ++i )
	result(i)*=double(b);
  return result;
}
template<class T>
DArray operator* (const T b, const DArray A) {return A*b;}

template<class T>
IArray operator* (const IArray A, const T b)
{
  IArray result=A;
  for( unsigned int i=0; i<A.size(); ++i )
	result(i)*=int(b);
  return result;
}
template<class T>
IArray operator* (const T b, const IArray A) {return A*b;}

class Map
{
  public:
	DMatrix T; // Transformation matrix
	DVector c; // centroid
	Map() {T.resize(2,2); c.resize(2); }
};

Map ArbitraryTriangleToStandardTriangle(DMatrix C);

double CCInt(DMatrix Values, DMatrix IntT);

IMatrix Cartan(char name, int d);
IMatrix Cartan(char name, int d, DMatrix &D);

IArray Gdual();
IArray Gdual(IVector &conjug);
IArray Gprimal();

DArray Gradient(DMatrix Values);

DMatrix IntLookup_deltoid(int N);

DMatrix IntLookup_triangle(int N);

DVector Interp(DMatrix Points, DMatrix Values);
DVector Interp(DMatrix Points, DMatrix Values, DMatrix C);

JArray JacobianMap(DArray phi);

double Surface_Area(DMatrix Values, DMatrix IntT);

CMatrix T_A2(CVector z, int N);

DMatrix TriangleMap(DMatrix f, JArray J);
DMatrix TriangleMap(DMatrix f, DMatrix J);

double TriCCInt(DMatrix Values, DMatrix C, DMatrix IntT);

DArray TriGrad(DMatrix Values, DMatrix C);

double TriSurf(DMatrix Values, DMatrix C, DMatrix IntT);

IArray Weyl(char name, int d);
IArray Weyl(IMatrix C);

IMatrix dualfudo(int N);
IMatrix dualfudo_sorted(int N);

IVector orbit_info(IArray G, IMatrix fudo, int N);
IVector orbit_info(IArray G, IMatrix fudo, int N, IMatrix &orbitnumber, IMatrix &orbitaction, IMatrix &isotropy);

IMatrix primalfudo(int N);

IVector triangle_interior_points(DMatrix points, DMatrix C);
IVector triangle_interior_points(DMatrix points, DMatrix C, IVector flags);
IVector triangle_interior_points(DMatrix points, DMatrix C, IVector flags, double tol);

/* some sorting routines */
template<class T>
void swap(T &a, T &b);
template<class T>
IVector rowsort(boost::numeric::ublas::matrix<T> &M, IVector sortindex);
template<class T>
IVector rowsort(boost::numeric::ublas::matrix<T> &M, IVector sortindex, double tol);
template<class T>
IVector columnsort(boost::numeric::ublas::matrix<T> &M, IVector sortindex);
template<class T>
IVector columnsort(boost::numeric::ublas::matrix<T> &M, IVector sortindex, double tol);
template<class T>
IVector uniq(boost::numeric::ublas::matrix<T> &M);
template<class T>
IVector uniq(boost::numeric::ublas::matrix<T> &M, double tol);
template<class T>
IVector uniq(boost::numeric::ublas::matrix<T> &M, IVector sortindex);
template<class T>
IVector uniq(boost::numeric::ublas::matrix<T> &M, IVector sortindex, double tol);

#ifdef mex_h
#include "array_conv.hpp"
#endif /* mex_h */

#endif /* __Chebyshev */
