/* Construct the dual fundamental domain for the A_2 root system.
 * Input:
 * 	N : Downscaling factor.
 * Output:
 * 	The dual fundamental domain for the A_2 root system (optionally sorted for use with Gradient()).
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include <boost/numeric/ublas/matrix_proxy.hpp>

IMatrix restricted_dualfudo(int N)
{
  int ndx=0;
  IMatrix fudo;
  fudo.resize(2,N*N); // a bit big but we'll truncate later
  for( int k2=0; k2<=N/3; ++k2 )
	for( int k1=0; k1<=N/2; ++k1)
	  if((k1+k2 <= N/2)&&(k2 <= k1)) {
		fudo(0,ndx) = k1;
		fudo(1,ndx) = k2;
		++ndx;
	  }
  fudo.resize(2,ndx,true);
  return fudo;
}

IMatrix restricted_dualfudo_sorted(int N)
{
  int ndx=0;
  IMatrix fudo;
  fudo.resize(2,N*N); // a bit big but we'll truncate later
  for( int k1=0; k1<=N/2; ++k1 )
	for( int j=0; j<=k1/2; ++j )
	  if( k1-j <= N/2 ) {
		fudo(0,ndx) = k1-j;
		fudo(1,ndx) = j;
		++ndx;
	  }
  fudo.resize(2,ndx,true);
  return fudo;
}

IMatrix dualfudo(int N)
{
  return restricted_dualfudo(N);
  int ndx=0;
  IMatrix fudo;
  fudo.resize(2,N*N); // a bit big but we'll truncate later
  for( int k2=0; k2<=N/3; ++k2 )
	for( int k1=0; k1<=N/2; ++k1)
	  if((2*k1+k2 <= N)&&(k2 <= k1)) {
		fudo(0,ndx) = k1;
		fudo(1,ndx) = k2;
		++ndx;
	  }
  fudo.resize(2,ndx,true);
  return fudo;
}

/* Constructing the dualfudo and then sorting it is too slow, so construct it already sorted for our purposes */
IMatrix dualfudo_sorted(int N)
{
  return restricted_dualfudo_sorted(N);
  int ndx=0;
  IMatrix fudo;
  fudo.resize(2,N*N); // a bit big but we'll truncate later
  for( int k1=0; k1<=2*N/3; ++k1 )
	for( int j=0; j<=k1/2; ++j )
	  if( 2*k1-j <= N ) {
		fudo(0,ndx) = k1-j;
		fudo(1,ndx) = j;
		++ndx;
	  }
  fudo.resize(2,ndx,true);
  return fudo;
}
