/* Row and column sorting routines for matrices.
 * Add explicit template instantiations as required.
 *
 * uniq() does a row sort then removes duplicate rows.
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include <boost/numeric/ublas/matrix_proxy.hpp>

template<class T>
void swap(T &a, T &b)
{
  T temp=a;
  a=b;
  b=temp;
}

template<class T>
IVector rowsort(boost::numeric::ublas::matrix<T> &M, IVector sortindex)
{
  return rowsort(M,sortindex,0.);
}

template<class T>
IVector rowsort(boost::numeric::ublas::matrix<T> &M, IVector sortindex, double tol)
{ // this is a gnomesort. It should be fast since input data is usually nearly sorted
  int i=1,j=2;
  IVector ndx;
  ndx.resize(M.size1());
  for( int k=0; k<(int)ndx.size(); ++k )
	ndx(k)=k;
  while( i<(int)M.size1() ) {
	bool do_swap=false;
	for( int k=0; k<(int)sortindex.size(); ++k ) {
	  if( fabs(M(i-1,sortindex(k))-M(i,sortindex(k)))<=tol )
		continue;
	  if( M(i-1,sortindex(k)) > M(i,sortindex(k)) ) {
		do_swap=true;
		break;
	  }
	  else
		break;
	}
	if( do_swap ) {
	  swap(row(M,i-1),row(M,i));
	  swap(ndx(i-1),ndx(i));
	  --i;
	  if( i==0 )
		i=1;
	}
	else {
	  i=j;
	  ++j;
	}
  }
  return ndx;
}

template<class T>
IVector columnsort(boost::numeric::ublas::matrix<T> &M, IVector sortindex)
{
  return columnsort(M,sortindex,0.);
}

template<class T>
IVector columnsort(boost::numeric::ublas::matrix<T> &M, IVector sortindex, double tol)
{
  IVector ndx;
  M=trans(M);
  ndx=rowsort(M,sortindex,tol);
  M=trans(M);
  return ndx;
}

template<class T>
IVector uniq(boost::numeric::ublas::matrix<T> &M)
{
  IVector sortindex;
  sortindex.resize(M.size2());
  for( int i=0; i<(int)sortindex.size(); ++i )
	sortindex(i)=i;
  return uniq(M,sortindex,0.);
}

template<class T>
IVector uniq(boost::numeric::ublas::matrix<T> &M, double tol)
{
  IVector sortindex;
  sortindex.resize(M.size2());
  for( int i=0; i<(int)sortindex.size(); ++i )
	sortindex(i)=i;
  return uniq(M,sortindex,tol);
}

template<class T>
IVector uniq(boost::numeric::ublas::matrix<T> &M, IVector sortindex)
{
  return uniq(M,sortindex,0.);
}

template<class T>
IVector uniq(boost::numeric::ublas::matrix<T> &M, IVector sortindex, double tol)
{
  int j=1;
  IVector ndx;
  ndx=rowsort(M,sortindex,tol);
  for( int i=1; i<(int)ndx.size(); ++i ) {
	for( int k=0; k<(int)sortindex.size(); ++k )
	  if( M(i,sortindex(k))-M(i-1,sortindex(k))>tol ) {
		ndx(j)=ndx(i);
		row(M,j++)=row(M,i);
		break;
	  }
	// trim the matrix
  }
  ndx.resize(j,true);
  M.resize(j,M.size2(),true);
  return ndx;
}

// explicit template instantiations, add more if needed
template IVector rowsort<double>(DMatrix &, IVector);
template IVector rowsort<int>(IMatrix &, IVector);
template IVector columnsort<double>(DMatrix &, IVector);
template IVector columnsort<int>(IMatrix &, IVector);
template IVector uniq<double>(DMatrix &);
template IVector uniq<int>(IMatrix &);
template IVector uniq<double>(DMatrix &, double);
template IVector uniq<int>(IMatrix &, double);
template IVector uniq<double>(DMatrix &, IVector);
template IVector uniq<int>(IMatrix &, IVector);
