/* Construct the primal and dual symmetry groups of the A_2 root system.
 * Input:
 * 	conjug (optional) : Determines if the symmetry is complex conj. or not (+-1).
 * Output:
 * 	The primal or dual symmetry group.
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"

IArray Gdual()
{
  IVector conjug;
  return Gdual(conjug);
}

IArray Gdual(IVector &conjug)
{
  // orbit: I,A',B',(AB)',(BA)',(ABA)',-I,-A',-B',-(AB)',-(BA)',-(ABA)'
  IArray G;

  // -1 for complex conjugation symmetry
  conjug.resize(12);
  for( int i=0; i<6; ++i ) {
	conjug(i)=1;
	conjug(i+6)=-1;
  }

  G=Gprimal();
  for( int i=0; i<6; ++i)
	G(i)=boost::numeric::ublas::trans(G(i));
  G.resize(12,true);
  for( int i=0; i<6; ++i)
	G(i+6) = -G(i);
  return G;
}

IArray Gprimal()
{
  // orbit: I,A,B,AB,BA,ABA
  IMatrix A(2,2),B(2,2),I;
  IArray G;

  I=boost::numeric::ublas::identity_matrix<int> (2);
  A(0,0)=1;A(0,1)=0;A(1,0)=1;A(1,1)=-1;
  B(0,0)=-1;B(0,1)=1;B(1,0)=0;B(1,1)=1;
  G.resize(6);
  G(0)=I;
  G(1)=A;
  G(2)=B;
  G(3)=prod(A,B);
  G(4)=prod(B,A);
  G(5)=prod(A,G(4));
  return G;
}
