/* Clenshaw-Curtis integration over the standard triangle using precomputed integrals of Chebyshev polynomials.
 * Input:
 *	Values : Values of the function on the Chebyshev extremal points.
 *	C : Triangle corners as a 2x3 matrix.
 *  IntT : Table of integrals of Chebyshev polynomials from IntLookup_triangle().
 * Output:
 *  The integral over the inscribed triangle.
 *
 * Brett Ryland, April 2009
 */

#include "Chebyshev.hpp"
#include <cmath>

double TriCCInt(DMatrix Values, DMatrix C, DMatrix IntT)
{
  Map map;
  map=ArbitraryTriangleToStandardTriangle(C);
  return CCInt(Values,IntT)*fabs(map.T(0,0)*map.T(1,1)-map.T(1,0)*map.T(0,1));
}
