/* Filter out the high frequency components of the solution.
 */

#include "Chebyshev.hpp"
#include <complex>
#include <fftw3.h>
#include <boost/numeric/ublas/matrix_proxy.hpp>

#define imod(x,y) ((x)-(y)*int(floor(double(x)/(y))))

DMatrix Filter(DMatrix in)
{
  int N;
  IMatrix g;
  CMatrix A,a;
  DMatrix out;
  fftw_plan plan,plan_inv;

  N=in.size1();
  if( in.size2()!=(unsigned)N ) {
	std::cerr << "Input array is the wrong size" << std::endl;
	exit(1);
  }

  g = dualfudo(N);

  a.resize(N,N/2+1);
  plan = fftw_plan_dft_r2c_2d(N, N, &(in.data()[0]), reinterpret_cast<fftw_complex*>(&(a.data()[0])), FFTW_ESTIMATE);
  fftw_execute(plan);

  A.resize(N,N/2+1);
  A.clear();
  // pseudo Fourier coefficients, A is a inside the dual fundamental domain and zero outside
  for( unsigned int j=0; j<g.size2(); ++j ) {
	A(g(0,j),g(1,j))=a(g(0,j),g(1,j));
	A(g(1,j),g(0,j))=a(g(1,j),g(0,j));
  }

  // apply symmetries (being careful of the reduced sizes of A)
  for( int j=0; j<(int)g.size2(); ++j ) {
	if( imod(g(0,j)+g(1,j),N)<N/2+1 )
	  A(imod(-g(0,j),N),imod(g(0,j)+g(1,j),N))=A(g(0,j),g(1,j));
	if( imod(-g(1,j),N)<N/2+1 )
	  A(imod(g(0,j)+g(1,j),N),imod(-g(1,j),N))=A(g(0,j),g(1,j));
	A(imod(-g(0,j)-g(1,j),N),g(0,j))=A(g(0,j),g(1,j));
	if( imod(-g(0,j)-g(1,j),N)<N/2+1 )
	  A(g(1,j),imod(-g(0,j)-g(1,j),N))=A(g(0,j),g(1,j));
	if( imod(-g(0,j),N)<N/2+1 )
	  A(imod(-g(1,j),N),imod(-g(0,j),N))=A(g(0,j),g(1,j));
	if( imod(g(1,j)+g(0,j),N)<N/2+1 )
	  A(imod(-g(1,j),N),imod(g(1,j)+g(0,j),N))=A(g(1,j),g(0,j));
	if( imod(-g(0,j),N)<N/2+1 )
	  A(imod(g(1,j)+g(0,j),N),imod(-g(0,j),N))=A(g(1,j),g(0,j));
	A(imod(-g(1,j)-g(0,j),N),g(1,j))=A(g(1,j),g(0,j));
	if( imod(-g(1,j)-g(0,j),N)<N/2+1 )
	  A(g(0,j),imod(-g(1,j)-g(0,j),N))=A(g(1,j),g(0,j));
	if( imod(-g(1,j),N)<N/2+1 )
	  A(imod(-g(0,j),N),imod(-g(1,j),N))=A(g(1,j),g(0,j));
  }

  // take the 2D IFFT to recover the filtered function
  out.resize(N,N);
  plan_inv = fftw_plan_dft_c2r_2d(N, N, reinterpret_cast<fftw_complex*>(&(A.data()[0])), &(out.data()[0]), FFTW_ESTIMATE);
  fftw_execute(plan_inv);
  out/=N*N; // normalise the FFT2

  // clean up
  fftw_destroy_plan(plan);
  fftw_destroy_plan(plan_inv);

  return out;
}
