/* Generate a matrix that fixes the points on the boundary and outside the domain.
 */

#include "Chebyshev_Spectral.hpp"

// boundary tolerance detection
#define tol 1e-10

DMatrix Gen_Fixed(DArray C, int N)
{
  using namespace boost::numeric::ublas;

  int I;
  double th1,th2,x,y,temp;
  IMatrix fudo;
  DMatrix F;
  Map map;

  fudo=primalfudo(N);
  F.resize(fudo.size2()*C.size(),fudo.size2()*C.size());
  F.clear();
  I=0;
  for( unsigned int tri=0; tri<C.size(); ++tri ) {
	for( unsigned int i=0; i<fudo.size2(); ++i ) {
	  // if i is inside the domain, set the diagonal entry to 1, otherwise leave it at 0
	  map=ArbitraryTriangleToStandardTriangle(C(tri));
	  th1=fudo(0,i)*2.*M_PI/double(N);
	  th2=fudo(1,i)*2.*M_PI/double(N);
	  x=(cos(th1)+cos(th2)+cos(th1-th2))/3.;
	  y=(sin(th1)-sin(th2)-sin(th1-th2))/3.;
	  temp=map.T(1,0)*x+map.T(1,1)*y+map.c(1);
	  x=map.T(0,0)*x+map.T(0,1)*y+map.c(0);
	  y=temp;
	  if( x>tol && x<1.-tol && y>tol && y<1.-tol )
		F(I,I)=1.;
	  ++I;
	}
  }
  return F;
}
