#include <iostream>
#include <cfloat>
#include "../Spectral/Chebyshev_Spectral.hpp"
#include <boost/numeric/ublas/io.hpp>
#include <boost/numeric/ublas/matrix_proxy.hpp>

#define imod(x,y) ((x)-(y)*(int)floor((double)(x)/(y)))
#define max(x,y) (x)>(y)?(x):(y)

DMatrix func(int N)
{
  double x,y;
  DVector theta;
  DMatrix Values;

  Values.resize(N,N);
  theta.resize(N);
  for( int i=0; i<N; ++i )
	theta(i)=i*2.*M_PI/N;
  for( int j=0; j<N; ++j)
	for( int i=0; i<N; ++i) {
	  x = (cos(theta(i)) + cos(theta(j)) + cos(theta(i)-theta(j)))/3.;
	  y = (sin(theta(i)) - sin(theta(j)) - sin(theta(i)-theta(j)))/3.;
	  if( fabs(x) < DBL_EPSILON )
		x=0.;
	  if( fabs(y) < DBL_EPSILON )
		y=0.;

	  Values(i,j) = exp(sin(2.*M_PI*x)*sin(2.*M_PI*y));
	}

  return Values;
}

DMatrix Coords(DMatrix C, IMatrix fudo, int N)
{
  double temp;
  DVector theta;
  DMatrix coords;
  Map map;

  map=ArbitraryTriangleToStandardTriangle(C);
  coords.resize(2,fudo.size2());
  theta.resize(N);
  for( int i=0; i<N; ++i )
	theta(i)=i*2.*M_PI/N;
  for( unsigned int i=0; i<fudo.size2(); ++i ) {
	coords(0,i) = (cos(theta(fudo(0,i))) + cos(theta(fudo(1,i))) + cos(theta(fudo(0,i))-theta(fudo(1,i))))/3.;
	coords(1,i) = (sin(theta(fudo(0,i))) - sin(theta(fudo(1,i))) - sin(theta(fudo(0,i))-theta(fudo(1,i))))/3.;
	if( fabs(coords(0,i)) < DBL_EPSILON )
	  coords(0,i)=0.;
	if( fabs(coords(1,i)) < DBL_EPSILON )
	  coords(1,i)=0.;
	temp=map.T(1,0)*coords(0,i)+map.T(1,1)*coords(1,i)+map.c(1);
	coords(0,i)=map.T(0,0)*coords(0,i)+map.T(0,1)*coords(1,i)+map.c(0);
	coords(1,i)=temp;
  }

  return coords;
}

int main(int argc, char** argv)
{
  using namespace boost::numeric::ublas;
  using std::cout;
  using std::cerr;
  using std::endl;
  int N,Ns,d;
  double lambda;
  IVector InnerPoints,LInnerPoints;
  IMatrix fudo,Lfudo;
  DVector Lvalues;
  DMatrix coords,Lcoords,Lambda;
  IArray W;
  DArray C,U;
  Overlap overlap;
  cout.precision(15);

  if( argc<3 ) {
	cerr << "3 input arguments are required: N Ns d" << endl;
	exit(1);
  }
  N=atoi(argv[1]);
  Ns=atoi(argv[2]);
  d=atoi(argv[3]);

  C.resize(2*d*d);
  for( int j1=0; j1<d; ++j1 )
	for( int j2=0; j2<d; ++j2 ) {
	  C(2*(j1*d+j2)).resize(2,3);
	  C(2*(j1*d+j2))(0,0)=(double)j2; C(2*(j1*d+j2))(0,1)=(double)j2; C(2*(j1*d+j2))(0,2)=(double)j2+1.;
	  C(2*(j1*d+j2))(1,0)=(double)j1; C(2*(j1*d+j2))(1,1)=(double)j1+1.; C(2*(j1*d+j2))(1,2)=(double)j1;
	  C(2*(j1*d+j2))/=(double)d;
	  C(2*(j1*d+j2)+1).resize(2,3);
	  C(2*(j1*d+j2)+1)(0,0)=(double)j2; C(2*(j1*d+j2)+1)(0,1)=(double)j2+1.; C(2*(j1*d+j2)+1)(0,2)=(double)j2+1.;
	  C(2*(j1*d+j2)+1)(1,0)=(double)j1+1.; C(2*(j1*d+j2)+1)(1,1)=(double)j1+1.; C(2*(j1*d+j2)+1)(1,2)=(double)j1;
	  C(2*(j1*d+j2)+1)/=(double)d;
	}
  U.resize(C.size());
  for( unsigned int i=0; i<C.size(); ++i )
	U(i).resize(N,N);
  overlap=Construct_Overlap(C, N);
  Lambda.resize(Ns,Ns);
  Lambda.clear();
  W=Weyl('A',2);
  fudo=primalfudo(N);
  Lfudo=primalfudo(Ns);
  Lcoords=Coords(C(0),Lfudo,Ns); // high res coordinates
  LInnerPoints=triangle_interior_points(Lcoords,C(0));
  for( unsigned int i=0; i<LInnerPoints.size(); ++i )
	column(Lcoords,i)=column(Lcoords,LInnerPoints(i));
  Lcoords.resize(2,LInnerPoints.size(),true); // just the ones not outside the triangle
  for( unsigned int tri=0; tri<C.size(); ++tri ) {
	coords=Coords(C(tri),fudo,N);
	InnerPoints=triangle_interior_points(coords,C(tri));
	for( unsigned int i=0; i<InnerPoints.size(); ++i ) {
	  IVector k(2);
	  // construct delta functions
	  for( unsigned int j=0; j<C.size(); ++j )
		U(j).clear();
	  k(0)=fudo(0,InnerPoints(i));
	  k(1)=fudo(1,InnerPoints(i));
	  for( unsigned int j=0; j<W.size(); ++j ) {
		IVector K(2);
		K=prod(trans(W(j)),k);
		K(0)=imod(K(0),N);
		K(1)=imod(K(1),N);
		U(tri)(K(0),K(1))=1.;
	  }
	  U=Smooth_Overlap(overlap, C, U);

	  // sample the function in one triangle on a higher resolution grid
	  Lvalues=Interp(Lcoords,U(0),C(0));
	  for( unsigned int j=0; j<LInnerPoints.size(); ++j )
		Lambda(Lfudo(0,LInnerPoints(j)),Lfudo(1,LInnerPoints(j)))+=fabs(Lvalues(j));
	}
  }
  lambda=0.;
  for( unsigned int j=0; j<LInnerPoints.size(); ++j )
	lambda=max(lambda,Lambda(Lfudo(0,LInnerPoints(j)),Lfudo(1,LInnerPoints(j))));
  cout << lambda << endl;
  return 0;
}
