function Deltoid_Lines(N)
% Deltoid_Lines(N)
% Plot the straight lines in the Deltoid corresponding to the lines
% in the \theta domain of the downscaled root lattice.
% Blue points: Chebyshev extremal points along even numbered lines
% Red points: Chebyshev extremal points along odd numbered lines
% Green points: Chebyshev roots along odd numbered lines
%
% Input:
% 	N (int) = the downscaling factor.

% Brett Ryland, 2008.

th = linspace (0, pi, N/2+1); % actually (0, 2*pi, N+1), but duplicity removes half of them
figure(1)
clf;
hold on;
for j = 0:N-1;
	[x, y] = deltocoord(th-pi*j/N, th+pi*j/N);
	if rem(j,2)==0
		plot(x, y, 'b-');
		plot(x, y, 'b.', 'MarkerSize', 20);
	else
		plot(x, y, 'b-');
		% plot(x, y, 'r.', 'MarkerSize', 20);
	end
end
axis square;
for j = 0:N-1;
	[x, y] = deltocoord(th-pi*j/N, th+pi*j/N);
	if rem(j,2)==1
		% uncomment the next line to show the Chebyshev roots along red lines.
		% plot((x(end)-x(1))/2*cos(2*[1/2:N/2-1/2]*pi/N)+(x(1)+x(end))/2, (y(end)-y(1))/2*cos(2*[1/2:N/2-1/2]*pi/N)+(y(1)+y(end))/2, 'g.');
	end
end
ezplot('1/3*(cos(2*t)+2*cos(t))','1/3*(-sin(2*t)+2*sin(t))',[0 2*pi])
h=get(gcf,'Children');
h2=get(h,'Children');
set(h2(1),'Color',[0 0 0]);
axis tight
axis square
title('Deltoid in (x_1,x_2)')
xlabel('x_1')
ylabel('x_2')
hold off
figure(2)
clf
hold on
plot([0 2*pi],[0 0],'k')
plot([0 -pi],[0 sqrt(3)*pi],'k')
plot([0 pi 0 0],1/sqrt(3)*pi*[0 1 2 0],'k')
for j=0:N/3
	plot(2*pi/N*[0		3/2*j],		sqrt(3)*pi/N*[j			j])
	plot(2*pi/N*[0		3/2*j],		sqrt(3)*pi/N*[2*N/3-j	2*N/3-j])
	plot(2*pi/N*[0		3/4*j],		sqrt(3)*pi/N*[2*j		j/2])
	plot(2*pi/N*[0		N/4-3/4*j],		sqrt(3)*pi/N*[2*j		N/2+j/2])
	plot(2*pi/N*[3/2*j	N/4+3/4*j],		sqrt(3)*pi/N*[2*N/3-j	N/6+j/2])
	plot(2*pi/N*[3/2*j	N/4+3/4*j],		sqrt(3)*pi/N*[j			N/2-j/2])
end
xlim([-2/3*pi 4/3*pi])
axis equal
title('Triangle in (\theta_1,\theta_2)')
text(4/3*pi-0.2,0.1,'\theta_1')
text(-2/3*pi+0.2,2/sqrt(3)*pi-0.1,'\theta_2')
xlabel('')
ylabel('')
hold off
