% Create a spectral element method on the unit square using multivariate Chebyshev polynomials
% Heat equation u_{t}=u_{xx}+u_{yy} with forward Euler time-stepping

% decompose the domain into triangles (with clockwise orientation)
d=3;
N=12; % try varying N and d such that 2*d^2*N^2 ~ 50000-80000
C=[];
for j1=0:d-1
  for j2=0:d-1
	C(:,:,2*(j1*d+j2)+1)=[j2 j2 j2+1;j1 j1+1 j1]/d;
	C(:,:,2*(j1*d+j2)+2)=[j2+1 j2+1 j2;j1+1 j1 j1+1]/d;
  end
end
if exist('P')~=1 | size(P,1)~=size(P,2) | size(P,1)~=2*d^2*size(primalfudo(N),2)
  disp('Generating projection matrix...')
  P=getProj(C,N);
  disp('... done.')
end
F=getFixed(C,N);

func='x.^2.*y.^2+1+exp(-10*((x-.5).^2+(y-.5).^2))';
% initial conditions, NxN for each triangle
u=zeros(N,N,size(C,3));
ux=zeros(size(u));
uy=zeros(size(u));
uxx=zeros(size(u));
uyy=zeros(size(u));
theta = [0:N-1]*2*pi/N;
X = reshape((cos(theta.'*ones(1,N))+cos(ones(N,1)*theta)+cos(theta.'*ones(1,N)-ones(N,1)*theta))/3,1,N*N);
Y = reshape((sin(theta.'*ones(1,N))-sin(ones(N,1)*theta)-sin(theta.'*ones(1,N)-ones(N,1)*theta))/3,1,N*N);
% set up grids on the triangles and evaluate the function on each domain to get some initial data
for j=1:size(C,3)
  [T,centroid]=ArbitraryTriangleToStandardTriangle(C(:,:,j));
  x=T(1,:)*[X;Y]+centroid(1);
  y=T(2,:)*[X;Y]+centroid(2);
  u(:,:,j)=reshape(eval(func),N,N);
end
u=VectorToArray(P*ArrayToVector(u,N,d),N,d);
u0=(eye(size(F))-F)*ArrayToVector(u,N,d);
clf;hold on;axis([-.2 1.2 -.2 1.2]);view([-10 50]);title('Initial conditions');rotate3d on
set(gcf,'DoubleBuffer','on');
for j=1:size(C,3)
  [T,centroid]=ArbitraryTriangleToStandardTriangle(C(:,:,j));
  x=reshape(T(1,:)*[X;Y],N,N)+centroid(1);
  y=reshape(T(2,:)*[X;Y],N,N)+centroid(2);
  surf(x,y,u(:,:,j));
end
drawnow
disp('press any key to continue')
pause
v=view();
% mov = avifile('movies/Heat_Equation.avi','compression','none','fps',30);
% for t=1:15
  % mov = addframe(mov,getframe(gcf));
% end

% dt=5*0.24/N^2/d^2; % near the largest stable timestep
dt=2e-4;
tmax=5e3;
for t=1:tmax
  for j=1:size(C,3)
	Du=TriGrad(u(:,:,j),C(:,:,j),N); % gradient
	ux(:,:,j)=real(Du(:,:,1));
	uy(:,:,j)=real(Du(:,:,2));
  end
  ux=VectorToArray(P*ArrayToVector(ux,N,d),N,d);
  uy=VectorToArray(P*ArrayToVector(uy,N,d),N,d);
  for j=1:size(C,3)
	Dux=TriGrad(ux(:,:,j),C(:,:,j),N); % uxx and uxy
	Duy=TriGrad(uy(:,:,j),C(:,:,j),N); % uyx and uyy
	uxx(:,:,j)=real(Dux(:,:,1));
	uyy(:,:,j)=real(Duy(:,:,2));
  end
  uxx=VectorToArray(P*ArrayToVector(uxx,N,d),N,d);
  uyy=VectorToArray(P*ArrayToVector(uyy,N,d),N,d);
  u=u+dt*(uxx+uyy);
  u=VectorToArray(u0+F*P*ArrayToVector(u,N,d),N,d);
  if (t<1500 & rem(t,10)==0) | (t>=1500 & rem(t,50)==0) % plot some graphs every now and then
	clf;hold on;axis([-.2 1.2 -.2 1.2]);view(v);title(sprintf('u at t=%3.2f (%3.1f%% dt=%g)',t*dt,100*t/tmax,dt));
	for j=1:size(C,3)
	  [T,centroid]=ArbitraryTriangleToStandardTriangle(C(:,:,j));
	  x=reshape(T(1,:)*[X;Y],N,N)+centroid(1);
	  y=reshape(T(2,:)*[X;Y],N,N)+centroid(2);
	  surf(x,y,u(:,:,j));
	end
	drawnow
	% mov = addframe(mov,getframe(gcf));
  end
end
% mov = close(mov);
figure;hold on;axis([-.2 1.2 -.2 1.2]);view(v);title(sprintf('u_{xx}+u_{yy} at t=%3.2f',t*dt));
for j=1:size(C,3)
  [T,centroid]=ArbitraryTriangleToStandardTriangle(C(:,:,j));
  x=reshape(T(1,:)*[X;Y],N,N)+centroid(1);
  y=reshape(T(2,:)*[X;Y],N,N)+centroid(2);
  surf(x,y,uxx(:,:,j)+uyy(:,:,j));
end
drawnow
