% A spectral method on a single deltoid.
% Forward Euler is applied to the heat equation u_t=u_xx+u_yy with the boundary being driven in an oscillatory manner.

N=18;
dt=1e-4;
tmax=2e3;
u=zeros(N,N);
uxx=zeros(size(u));
uyy=zeros(size(u));
theta = [0:N-1]*2*pi/N;
x = (cos(theta.'*ones(1,N))+cos(ones(N,1)*theta)+cos(theta.'*ones(1,N)-ones(N,1)*theta))/3;
y = (sin(theta.'*ones(1,N))-sin(ones(N,1)*theta)-sin(theta.'*ones(1,N)-ones(N,1)*theta))/3;
u=eval('1+x-y+exp(-10*((x+.1).^2+(y-.1).^2))-2*exp(-100*((x+.1).^2+(y-.1).^2))');
% u=eval('heaviside(.1-x).*heaviside(.1+x).*heaviside(.1-y).*heaviside(.1+y)');

% points on the outer edge
W=Weyl(Cartan('A',2));
[X,ndx1]=max(x);[X,ndx2]=min(x);
if rem(N,2)==0
  [X,ndx1(N/2+1)]=max(y(:,N/2+1)); [X,ndx2(N/2+1)]=min(y(:,N/2+1));
end
ndx=[];for j=1:6;ndx=[ndx mod(W(:,:,j).'*[ndx1-1 ndx2-1;0:N-1 0:N-1],N)+1];end;ndx=uniq(ndx.').'; boundary=(ndx(2,:)-1)*N+ndx(1,:);

% points on the triangle
% boundary=find((abs(x+1/3)<1e-10)|(abs(-sqrt(3)*y-x+2/3)<1e-10)|(abs(sqrt(3)*y-x+2/3)<1e-10));

% points outside the triangle
% boundary=find((x+1/3<1e-10)|(-sqrt(3)*y-x+2/3<1e-10)|(sqrt(3)*y-x+2/3<1e-10));

u0=u(boundary);
% U=nan(size(u));
% in=find((x+1/3>-1e-10)&(-sqrt(3)*y-x+2/3>-1e-10)&(sqrt(3)*y-x+2/3>-1e-10)); U(in)=u(in);
% fig=gcf;surf(x,y,U);axis([-1/3 2/3 -1/sqrt(3) 1/sqrt(3) 0 2]);view([-93 20]);rotate3d on;title('Initial conditions');drawnow
surf(x,y,u);view([-93 20]);rotate3d on;title('Initial conditions');drawnow
% set(fig,'DoubleBuffer','on');
% mov = avifile('movies/Spectral_method.avi','compression','none');
% for t=1:15
  % mov = addframe(mov,getframe(gcf));
% end
tol=1e-15;
for t=1:tmax
  Du=Gradient(u,N); % gradient
  Dux=Gradient(real(Du(:,:,1)),N); % uxx and uxy
  Duy=Gradient(real(Du(:,:,2)),N); % uyx and uyy
  uxx=real(Dux(:,:,1)); % discard imaginary components, they should be zero
  uyy=real(Duy(:,:,2)); % discard imaginary components, they should be zero
  u=u+dt*(uxx+uyy);
  u(boundary)=u0+sin(50*t*dt)*cos(pi*x(boundary)).*cos(pi*y(boundary));
  % u(boundary)=u0;
  % if ismember(t,round(logspace(0,log10(tmax),200)))
  if rem(t,20)
	% U(in)=u(in);
	% figure(fig);surf(x,y,U);axis([-1/3 2/3 -1/sqrt(3) 1/sqrt(3) 0 2]);view([-93 20]);title(sprintf('u at t=%d',t*dt));drawnow
	surf(x,y,u);view([-93 20]);title(sprintf('u at t=%d',t*dt));drawnow
	% mov = addframe(mov,getframe(gcf));
  end
  if max(max(abs(dt*(uxx+uyy))))<tol
	break
  end
end
% mov = close(mov);
