% Create a spectral element method on the 2-torus using multivariate Chebyshev polynomials
% Leapfrog is applied to the wave equation with interpolation providing communication between triangular domains.

% decompose the domain into triangles (with clockwise orientation)
d=3;
N=18; % try varying N and d such that 2*d^2*N^2 ~ 50000-80000
C=[];
for j1=0:d-1
  for j2=0:d-1
	C(:,:,2*(j1*d+j2)+1)=[j2 j2 j2+1;j1 j1+1 j1]/d;
	C(:,:,2*(j1*d+j2)+2)=[j2 j2+1 j2+1;j1+1 j1+1 j1]/d;
  end
end
[overlap,edges,corners]=Construct_Overlap(C,N);

func='(exp(sin(2*pi*x-pi/2)+1)-1).*(exp(sin(2*pi*y-pi/2)+1)-1)/16';
dfunc='0.*x.*y';
% initial conditions, NxN for each triangle
u=zeros(N,N,size(C,3));
v=zeros(N,N,size(C,3));
uxx=zeros(size(u));
uyy=zeros(size(u));
theta = [0:N-1]*2*pi/N;
x = reshape(1/3*(exp(i*theta.'*ones(1,N)) + exp(-i*ones(N,1)*theta) + exp(-i*(theta.'*ones(1,N)-ones(N,1)*theta))),1,N*N);
Y=imag(x); X=real(x);
% set up grids on the triangles and evaluate the function on each domain to get some initial data
for j=1:size(C,3)
  [T,centroid]=ArbitraryTriangleToStandardTriangle(C(:,:,j));
  x=T(1,:)*[X;Y]+centroid(1);
  y=T(2,:)*[X;Y]+centroid(2);
  u(:,:,j)=reshape(eval(func),N,N);
  v(:,:,j)=reshape(eval(dfunc),N,N);
end
u=Smooth_Overlap(overlap,edges,corners,C,u);
fig=figure;clf;hold on;axis([-.2 1.2 -.2 1.2 -2 2]);view([-10 50]);rotate3d on;title('Initial conditions');
set(fig,'DoubleBuffer','on');
for j=1:size(C,3)
  [T,centroid]=ArbitraryTriangleToStandardTriangle(C(:,:,j));
  x=reshape(T(1,:)*[X;Y],N,N)+centroid(1);
  y=reshape(T(2,:)*[X;Y],N,N)+centroid(2);
  surf(x,y,u(:,:,j));
end
drawnow
% mov = avifile('movies/Spectral_element_method.avi','compression','none','fps',15);
% for t=1:15
  % mov = addframe(mov,getframe(gcf));
% end

% dt=5*0.24/N^2/d^2; % near the largest stable timestep
dt=5e-4;
tmax=1e3;
% initial half-step in v and full step in u
for j=1:size(C,3)
  Du=TriGrad(u(:,:,j),C(:,:,j),N); % gradient
  Dux=TriGrad(real(Du(:,:,1)),C(:,:,j),N); % uxx and uxy
  Duy=TriGrad(real(Du(:,:,2)),C(:,:,j),N); % uyx and uyy
  v(:,:,j)=v(:,:,j)+dt/2*real(Dux(:,:,1));
  v(:,:,j)=v(:,:,j)+dt/2*real(Duy(:,:,2));
end
u=u+dt*v;
for t=2:tmax
  for j=1:size(C,3)
	Du=TriGrad(u(:,:,j),C(:,:,j),N); % gradient
	Dux=TriGrad(real(Du(:,:,1)),C(:,:,j),N); % uxx and uxy
	Duy=TriGrad(real(Du(:,:,2)),C(:,:,j),N); % uyx and uyy
	v(:,:,j)=v(:,:,j)+dt*real(Dux(:,:,1));
	v(:,:,j)=v(:,:,j)+dt*real(Duy(:,:,2));
  end
  u=u+dt*v;
  u=Smooth_Overlap(overlap,edges,corners,C,u);
  if rem(t,10)==0 % plot some graphs every now and then
	clf;hold on;axis([-.2 1.2 -.2 1.2 -2 2]);view([-10 50]);rotate3d on;title(sprintf('u at t=%3.2f (%3.1f%% dt=%g)',t*dt,100*t/tmax,dt));
	for j=1:size(C,3)
	  [T,centroid]=ArbitraryTriangleToStandardTriangle(C(:,:,j));
	  x=reshape(T(1,:)*[X;Y],N,N)+centroid(1);
	  y=reshape(T(2,:)*[X;Y],N,N)+centroid(2);
	  surf(x,y,u(:,:,j));
	end
	drawnow
	% mov = addframe(mov,getframe(gcf));
  end
end
% final half-step in v
for j=1:size(C,3)
  Du=TriGrad(u(:,:,j),C(:,:,j),N); % gradient
  Dux=TriGrad(real(Du(:,:,1)),C(:,:,j),N); % uxx and uxy
  Duy=TriGrad(real(Du(:,:,2)),C(:,:,j),N); % uyx and uyy
  v(:,:,j)=v(:,:,j)+dt/2*real(Dux(:,:,1));
  v(:,:,j)=v(:,:,j)+dt/2*real(Duy(:,:,2));
end
% mov = close(mov);
