% Create a spectral element method on the 2-torus using multivariate Chebyshev polynomials
% Wave equation with implicit midpoint time-stepping

% decompose the domain into triangles (with clockwise orientation)
d=3;
N=18; % try varying N and d such that 2*d^2*N^2 ~ 50000-80000
C=[];
for j1=0:d-1
  for j2=0:d-1
	C(:,:,2*(j1*d+j2)+1)=[j2 j2 j2+1;j1 j1+1 j1]/d;
	C(:,:,2*(j1*d+j2)+2)=[j2+1 j2+1 j2;j1+1 j1 j1+1]/d;
  end
end
if exist('P')~=1 | size(P,1)~=size(P,2) | size(P,1)~=2*d^2*size(primalfudo(N),2)
  disp('Generating projection matrix...')
  P=getProj(C,N);
  disp('... done.')
end
% F=getFixed(C,N);

func='(exp(sin(2*pi*x-pi/2)+1)-1).*(exp(sin(2*pi*y-pi/2)+1)-1)/16';
dfunc='0.*x.*y';
% initial conditions, NxN for each triangle
u=zeros(N,N,size(C,3));
v=zeros(N,N,size(C,3));
ux=zeros(size(u));
uy=zeros(size(u));
uxx=zeros(size(u));
uyy=zeros(size(u));
theta = [0:N-1]*2*pi/N;
x = reshape(1/3*(exp(i*theta.'*ones(1,N)) + exp(-i*ones(N,1)*theta) + exp(-i*(theta.'*ones(1,N)-ones(N,1)*theta))),1,N*N);
Y=imag(x); X=real(x);
% set up grids on the triangles and evaluate the function on each domain to get some initial data
for j=1:size(C,3)
  [T,centroid]=ArbitraryTriangleToStandardTriangle(C(:,:,j));
  x=T(1,:)*[X;Y]+centroid(1);
  y=T(2,:)*[X;Y]+centroid(2);
  u(:,:,j)=reshape(eval(func),N,N);
  v(:,:,j)=reshape(eval(dfunc),N,N);
end
u=VectorToArray(P*ArrayToVector(u,N,d),N,d);
% u0=(eye(size(F))-F)*ArrayToVector(u,N,d);
v=VectorToArray(P*ArrayToVector(v,N,d),N,d);
% v0=(eye(size(F))-F)*ArrayToVector(v,N,d);
clf;hold on;axis([-.2 1.2 -.2 1.2 -2 2]);view([-10 50]);title('Initial conditions');
set(gcf,'DoubleBuffer','on');
for j=1:size(C,3)
  [T,centroid]=ArbitraryTriangleToStandardTriangle(C(:,:,j));
  x=reshape(T(1,:)*[X;Y],N,N)+centroid(1);
  y=reshape(T(2,:)*[X;Y],N,N)+centroid(2);
  surf(x,y,u(:,:,j));
end
drawnow
% mov = avifile('movies/Wave_Equation_IMP.avi','compression','none','fps',30);
% for t=1:15
  % mov = addframe(mov,getframe(gcf));
% end

dt=2e-4;
tmax=5e3;
for t=1:tmax
  converge=1000;converge_last=converge+1;
  newu=u;newv=v;
  for j=1:size(C,3)
	Du=TriGrad(u(:,:,j),C(:,:,j),N); % gradient
	ux(:,:,j)=real(Du(:,:,1));
	uy(:,:,j)=real(Du(:,:,2));
  end
  ux=VectorToArray(P*ArrayToVector(ux,N,d),N,d);
  uy=VectorToArray(P*ArrayToVector(uy,N,d),N,d);
  for j=1:size(C,3)
	Dux=TriGrad(ux(:,:,j),C(:,:,j),N); % uxx and uxy
	Duy=TriGrad(uy(:,:,j),C(:,:,j),N); % uyx and uyy
	uxx(:,:,j)=real(Dux(:,:,1));
	uyy(:,:,j)=real(Duy(:,:,2));
  end
  uxx=VectorToArray(P*ArrayToVector(uxx,N,d),N,d);
  uyy=VectorToArray(P*ArrayToVector(uyy,N,d),N,d);
  del2u=uxx+uyy;
  while(converge<converge_last)
	converge_last=converge;
	lastu=newu;lastv=newv;
	for j=1:size(C,3)
	  Du=TriGrad(newu(:,:,j),C(:,:,j),N); % gradient
	  ux(:,:,j)=real(Du(:,:,1));
	  uy(:,:,j)=real(Du(:,:,2));
	end
	ux=VectorToArray(P*ArrayToVector(ux,N,d),N,d);
	uy=VectorToArray(P*ArrayToVector(uy,N,d),N,d);
	for j=1:size(C,3)
	  Dux=TriGrad(ux(:,:,j),C(:,:,j),N); % uxx and uxy
	  Duy=TriGrad(uy(:,:,j),C(:,:,j),N); % uyx and uyy
	  uxx(:,:,j)=real(Dux(:,:,1));
	  uyy(:,:,j)=real(Duy(:,:,2));
	end
	uxx=VectorToArray(P*ArrayToVector(uxx,N,d),N,d);
	uyy=VectorToArray(P*ArrayToVector(uyy,N,d),N,d);
	newv=v+dt/2*(uxx+uyy+del2u);
	% newv=VectorToArray(v0+F*P*ArrayToVector(newv,N,d),N,d);
	newv=VectorToArray(P*ArrayToVector(newv,N,d),N,d);
	newu=u+dt/2*(newv+v);
	% newu=VectorToArray(u0+F*P*ArrayToVector(newu,N,d),N,d);
	newu=VectorToArray(P*ArrayToVector(newu,N,d),N,d);
	converge=max(max(max(abs(newu-lastu))))+max(max(max(abs(newv-lastv))));
  end
  if(converge > 1000) % incase of blowup
	break
  end
  u=newu;
  v=newv;
  if rem(t,10)==0 % plot some graphs every now and then
	vw=view();clf;hold on;axis([-.2 1.2 -.2 1.2 -2 2]);view(vw);title(sprintf('u at t=%3.2f (%3.1f%% dt=%g)',t*dt,100*t/tmax,dt));
	for j=1:size(C,3)
	  [T,centroid]=ArbitraryTriangleToStandardTriangle(C(:,:,j));
	  x=reshape(T(1,:)*[X;Y],N,N)+centroid(1);
	  y=reshape(T(2,:)*[X;Y],N,N)+centroid(2);
	  surf(x,y,u(:,:,j));
	end
	drawnow
	% mov = addframe(mov,getframe(gcf));
  end
end
% mov = close(mov);
