function [I,T]=demo_deltoid_run(Nmax,iters)
% demo for using Chebyshev polynomials on a deltoid
IntT = IntLookup_deltoid(Nmax);
func='exp(sin(x).*sin(y))';
gradx=inline('cos(x).*sin(y).*exp(sin(x).*sin(y))','x','y');
grady=inline('sin(x).*cos(y).*exp(sin(x).*sin(y))','x','y');
exact_int=CCInt(func,Nmax,IntT);
theta = [0:Nmax-1]*2*pi/Nmax;
x = 1/3*(exp(i*theta.'*ones(1,Nmax)) + exp(-i*ones(Nmax,1)*theta) + exp(-i*(theta.'*ones(1,Nmax)-ones(Nmax,1)*theta)));
y=imag(x); % switch to real and imaginary coordinates
x=real(x);
S=sqrt(1+gradx(x,y).^2+grady(x,y).^2);
[exact_surface_area]=CCInt(S,Nmax,IntT);

I=zeros(4,Nmax);
T=ones(3,Nmax);
% CCInt timing and accuracy
for j=1:iters
  for N=1:Nmax
	tic;
	I(1,N)=abs(CCInt(func,N,IntT)-exact_int);
	T(1,N)=min(toc,T(1,N));
  end;
end
% Surface_Area timing and accuracy
for j=1:iters
  for N=1:Nmax
	tic;
	I(2,N)=abs(Surface_Area(func,N,IntT)-exact_surface_area);
	T(2,N)=min(toc,T(2,N));
  end;
end
% Gradient timing and accuracy
for j=1:iters
  for N=1:Nmax
	theta = [0:N-1]*2*pi/N;
	x = 1/3*(exp(i*theta.'*ones(1,N)) + exp(-i*ones(N,1)*theta) + exp(-i*(theta.'*ones(1,N)-ones(N,1)*theta)));
	y=imag(x); % switch to real and imaginary coordinates
	x=real(x);
	tic;
	D=Gradient(func,N);
	T(3,N)=min(toc,T(3,N));
	I(3:4,N)=[norm(D(:,:,1)-gradx(x,y))/norm(gradx(x,y)) norm(D(:,:,2)-grady(x,y))/norm(grady(x,y))];
  end;
end
