% test for using Chebyshev polynomials on a nonlinearly deformed triangle
% this was used to generate Figure 9 in "On Multivariate Chebyshev Polynomials and Spectral Approximations on Triangles"
Nmax=100; % maximum value of N
iters=5; % number of iterations to average timing over (only actually used in the MEX version)
[I,T]=demo_nonlinear_triangle_run(Nmax,iters);

x1=[-1/3;-1/sqrt(3);1];x2=[-1/3;1/sqrt(3);1];x3=[2/3;0;1];
R=1;
a=acos(dot(x1,x2)/norm(x1)/norm(x2));
b=acos(dot(x1,x3)/norm(x1)/norm(x3));
c=acos(dot(x2,x3)/norm(x2)/norm(x3));
A=acos((cos(a)-cos(b)*cos(c))/sin(b)/sin(c));
B=acos((cos(b)-cos(a)*cos(c))/sin(a)/sin(c));
C=acos((cos(c)-cos(a)*cos(b))/sin(a)/sin(b));
area=R^2*(A+B+C-pi);
% area=TriSurf('1./sqrt(x.^2+y.^2+1)',[0 0 1;0 1 0],Nmax,IntT);

clf;
[ax,h1,h2]=plotyy([1:Nmax],abs((I-area)/area),[1:Nmax],T,'semilogy','plot');
set(ax(1),'xlim',[0 Nmax],'ylim',[1e-16 1e0],'ytick',[1e-16 1e-14 1e-12 1e-10 1e-8 1e-6 1e-4 1e-2 1e0]);
axes(ax(2)); hold on;
f=inline('a(1)*x.^2+a(2)*(x.*log(x)).^2','a','x');a=lsqcurvefit(f,[0 0],[1:Nmax].',T,[0 0]);
plot([1:Nmax],a(1)*[1:Nmax].^2+a(2)*([1:Nmax].*log([1:Nmax])).^2,'k');
[B,Bi,R,Ri,S]=regress(T,[ones(Nmax,1) a(1)*[1:Nmax].'.^2+a(2)*([1:Nmax].'.*log([1:Nmax].')).^2]);
text(Nmax*0.65,max(T)*0.85,sprintf('y = a_1N^2+a_2(NlogN)^2\na_1 = %.3d\na_2 = %.3d\nR = %.5f',a(1),a(2),sqrt(S(1))));
f=inline('a(1)*x.^2','a','x');a=lsqcurvefit(f,[0 0],[1:Nmax].',T); plot([1:Nmax],a(1)*[1:Nmax].^2,'r');
[B,Bi,R,Ri,S]=regress(T,[ones(Nmax,1) a(1)*[1:Nmax].'.^2]);
h=text(Nmax*0.40,max(T)*0.87,sprintf('y = aN^2\na = %.3d\nR = %.5f',a(1),sqrt(S(1))));set(h,'color','r');
set(ax(2),'xlim',[0 Nmax],'ylim',[0 max(T)],'ytick',[0:max(T)/8:max(T)]);
title(sprintf('Spectral convergence of surface area of a spherical triangle'));
xlabel('N'); ylabel(ax(1),'Relative error');ylabel(ax(2),'Timing in seconds');
legend(ax(1),'off'); legend(ax(2),'off');
