#include <iostream>
#include <cfloat>
#include <mex.h> // mex.h must be included before Chebyshev.hpp
#include "../Spectral/Chebyshev_Spectral.hpp"
#include <boost/numeric/ublas/io.hpp>
#include <sys/time.h>

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
  using namespace std;
  int Nmax,iters,lin;
  DVector theta;
  DVector I,T;
  DMatrix x,y,f,IntT,F;
  DArray phi;
  JArray J;
  Map map;
  struct timeval t1,t2;

  if (nrhs != 2) {
	mexErrMsgTxt("Two input argument required (Nmax,iters).");
  }

  Nmax=(int)(*mxGetPr(prhs[0]));
  iters=(int)(*mxGetPr(prhs[1]));
  IntT=IntLookup_triangle(Nmax);
  I.resize(Nmax);
  T.resize(Nmax);


  for( int N=1; N<=Nmax; ++N ) {
	theta.resize(N);
	for( int i=0; i<N; ++i )
	  theta(i)=i*2.*M_PI/N;
	x.resize(N,N);
	y.resize(N,N);
	for( int j=0; j<N; ++j)
	  for( int i=0; i<N; ++i) {
		double temp;
		x(i,j) = (cos(theta(i)) + cos(theta(j)) + cos(theta(i)-theta(j)))/3.;
		y(i,j) = (sin(theta(i)) - sin(theta(j)) - sin(theta(i)-theta(j)))/3.;
		if( fabs(x(i,j)) < DBL_EPSILON )
		  x(i,j)=0.;
		if( fabs(y(i,j)) < DBL_EPSILON )
		  y(i,j)=0.;
	  }
	phi.resize(2);
	phi(0).resize(N,N);
	phi(1).resize(N,N);
	f.resize(N,N);

	// initialise the phi map and f function here.
	for( int i=0; i<N; ++i )
	  for( int j=0; j<N; ++j ) {
		double r=sqrt(x(i,j)*x(i,j)+y(i,j)*y(i,j)+1.);
		phi(0)(i,j)=x(i,j)/r;
		phi(1)(i,j)=y(i,j)/r;
		f(i,j)=1./r;
	  }

	// perform the surface area calculation
	T(N-1)=10.;
	J=JacobianMap(phi);
	for( int j=0; j<iters; ++j ) {
	  gettimeofday(&t1,0);
	  F=TriangleMap(f,J);
	  I(N-1)=CCInt(F,IntT);
	  gettimeofday(&t2,0);
	  T(N-1)=min(double(t2.tv_sec-t1.tv_sec)+double(t2.tv_usec-t1.tv_usec)*1e-6,T(N-1));
	}
  }
  array_conv(I,&plhs[0]);
  array_conv(T,&plhs[1]);
}
