% timing and accuracy demonstration for functions on the triangle
% this was used to generate Figures 7 & 8 in "On Multivariate Chebyshev Polynomials and Spectral Approximations on Triangles"
Nmax = 100;
Navg = 2; % number of loops to average over
disp('computing errors and timing of TriCCInt, TriSurf and TriGrad routines averaged over several runs for the triangle with corners [0 0 1;0 1 0]')
[I,T]=demo_triangle_run(Nmax,Navg);
I(isnan(I))=0;
figure(1);clf
[ax,h1,h2]=plotyy([1:Nmax],I(1,:),[1:Nmax],T(1,:),'semilogy','plot');
set(ax(1),'xlim',[0 Nmax],'ylim',[1e-16 1e0],'ytick',[1e-16 1e-14 1e-12 1e-10 1e-8 1e-6 1e-4 1e-2 1e0]);
axes(ax(2)); hold on
f=inline('a(1)*x.^2','a','x');a=lsqcurvefit(f,[0 0],[1:Nmax],T(1,:)); plot([1:Nmax],a(1)*[1:Nmax].^2,'k');
[B,Bi,R,Ri,S]=regress(T(1,:).',[ones(Nmax,1) a(1)*[1:Nmax].'.^2]);
text(Nmax*0.65,max(T(1,:))*0.87,sprintf('y = aN^2\na = %.3d\nR = %.5f',a(1),sqrt(S(1))));
set(ax(2),'xlim',[0 Nmax],'ylim',[0 max(T(1,:))],'ytick',[0:max(T(1,:))/8:max(T(1,:))]);
title(sprintf('Spectral convergence of integration of exp(sin(x)sin(y))'));
xlabel('N');ylabel(ax(1),'Relative error');ylabel(ax(2),'Timing in seconds');
hold off
figure(2);clf
[ax,h1,h2]=plotyy([1:Nmax],I(2,:),[1:Nmax],T(2,:),'semilogy','plot');
set(ax(1),'xlim',[0 Nmax],'ylim',[1e-16 1e0],'ytick',[1e-16 1e-14 1e-12 1e-10 1e-8 1e-6 1e-4 1e-2 1e0]);
axes(ax(2)); hold on
f=inline('a(1)*x.^2','a','x');a=lsqcurvefit(f,[0 0],[1:Nmax],T(2,:)); plot([1:Nmax],a(1)*[1:Nmax].^2,'k');
[B,Bi,R,Ri,S]=regress(T(2,:).',[ones(Nmax,1) a(1)*[1:Nmax].'.^2]);
text(Nmax*0.65,max(T(2,:))*0.87,sprintf('y = aN^2\na = %.3d\nR = %.5f',a(1),sqrt(S(1))));
set(ax(2),'xlim',[0 Nmax],'ylim',[0 max(T(2,:))],'ytick',[0:max(T(2,:))/8:max(T(2,:))]);
title(sprintf('Spectral convergence of surface area of exp(sin(x)sin(y))'));
xlabel('N');ylabel(ax(1),'Relative error');ylabel(ax(2),'Timing in seconds');
hold off
figure(3);clf
[ax,h1,h2]=plotyy([1:Nmax],I([3 4],:),[1:Nmax],T(3,:),'semilogy','plot');
axes(ax(1)); hold on; plot(ax(1),[1:Nmax],eps*[1:Nmax].^2,'r');
set(ax(1),'xlim',[0 Nmax],'ylim',[1e-16 max(1e0,max(max(I(3:4,:))))],'ytick',[1e-16 1e-14 1e-12 1e-10 1e-8 1e-6 1e-4 1e-2 1e0]);
h=text(10,1e-12,'2.22e-16N^2');set(h,'color','r');
axes(ax(2)); hold on
f=inline('a(1)*x.^2','a','x');a=lsqcurvefit(f,[0 0],[1:Nmax],T(3,:)); plot([1:Nmax],a(1)*[1:Nmax].^2,'k');
[B,Bi,R,Ri,S]=regress(T(3,:).',[ones(Nmax,1) a(1)*[1:Nmax].'.^2]);
text(Nmax*0.65,max(T(3,:))*0.87,sprintf('y = aN^2\na = %.3d\nR = %.5f',a(1),sqrt(S(1))));
set(ax(2),'xlim',[0 Nmax],'ylim',[0 max(T(3,:))],'ytick',[0:max(T(3,:))/8:max(T(3,:))]);
set(h1(2),'color',[0 0 0]);
set(h2,'color',[0 0.5 0]);
set(get(ax(2),'ylabel'),'color',[0 0.5 0]);
set(ax(2),'ycolor',[0 0.5 0]);
title(sprintf('Spectral convergence of \\nabla_{x}exp(sin(x)sin(y))'));
xlabel('N');ylabel(ax(1),'L_2 norm of the absolute error');ylabel(ax(2),'Timing in seconds');
hold off
