function [T,centroid]=ArbitraryTriangleToStandardTriangle(C)
% [T,centroid]=ArbitraryTriangleToStandardTriangle(C)
% map any triangle to the standard triangle to be used with the multivariate Chebyshev routines
%
% Input:
%	C (2x3 double) = corners of the triangle
%
% Output:
%	T (2x2 double) = transformation matrix between the triangles, C-centroid = T*[-1/3 -1/3 2/3;-1/sqrt(3) 1/sqrt(3) 0];
%		note that T is also the Jacobian of the transformation and that inv(T).' is used to convert the result of Gradient
%	centroid (2x1 double) = the centroid of the triangle

centroid=sum(C,2)/3;
v1=C(:,3)-centroid;
v1=v1*3/2;
v2=C(:,2)-C(:,1);
v2=v2/2*sqrt(3);
T=[v1 v2];
