function [C,D] = Cartan(name,d)
% [C,D] = Cartan(name,d)
% Calculate the Cartan matrix for a specified root system.
%
% Input:
%	name (char) = specifies which class of root systems is to be used.
%	d (int) = specifies the dimension of the root system.
% Output:
%	C (d*d double) = Cartan matrix
%	D (d*d double) = a diagonal matrix such that inv(D)*C*D is symmmetric

% Brett Ryland, March 2010.

switch name
case 'A'
	C = diag(2*ones(1,d),0)-diag(ones(1,d-1),1)-diag(ones(1,d-1),-1);
	D = 2*eye(d);
case 'B'
	if d < 2
		error('Dimension must be > 1 .');
	end
	C = diag(2*ones(1,d),0)-diag(ones(1,d-1),1)-diag(ones(1,d-1),-1);
	C(end-1,end) = -2;
	D = 2*eye(d);
	D(end,end) = sqrt(2);
case 'C'
	if d < 2
		error('Dimension must be > 1 .');
	end
	C = diag(2*ones(1,d),0)-diag(ones(1,d-1),1)-diag(ones(1,d-1),-1);
	C(end,end-1) = -2;
	D = sqrt(2)*eye(d);
	D(end,end) = (2);
case 'D'
	if d < 4
		error('Dimension must be > 3 .');
	end
	C = diag(2*ones(1,d),0)-diag(ones(1,d-1),1)-diag(ones(1,d-1),-1);
	C(end-1,end) = 0;
	C(end,end-1) = 0;
	C(end-2,end) = -1;
	C(end,end-2) = -1;
	D = 2*eye(d);
case 'E'
	if (d < 6) | (d > 8)
		error('Dimension must be 6, 7 or 8.'); 
	end
	C = diag(2*ones(1,d),0)-diag(ones(1,d-1),1)-diag(ones(1,d-1),-1);
	C(end-1,end) = 0;
	C(end,end-1) = 0;
	C(3,end) = -1;
	C(end,3) = -1;
	D = 2*eye(d);
case 'F'
	if d ~= 4
		error('Dimension must be 4.');
	end
	C = [2 -1 0 0; -1 2 -2 0; 0 -1 2 -1; 0 0 -1 2];
	D = [];
case 'G'
	if d ~= 2
		error('Dimension must be 2.');
	end
	C = [2, -1; -3 2];
	D = diag([2/sqrt(3),2]);
otherwise
	error('Unknown Cartan matrix');
end
