function [Jacobian, invJacobian] = Chebyshev_Jacobian( root_system, dimension )
% [Jacobian, invJacobian] = Chebyshev_Jacobian( root_system, dimension )
% calculate the Jacobian of the map between the deltoid and the triangle in \theta coordinates
%
% Input:
%	root_system (string) in {'A', 'B', 'C', 'D', 'E', 'F', 'G'} indicating the type of root system
%	dimension (int) = the dimension of space < 9
% e.g., for the A_2 root system, root_system = 'A', dimension = 2
%
% Output:
%	Jacobian (dimension,dimension) = Jacobian matrix
%	invJacobian (dimension,dimension) = inverse of the Jacobian matrix

% Brett Ryland, Nov 2007

syms t1 t2 t3 t4 t5 t6 t7 t8
t = [t1; t2; t3; t4; t5; t6; t7; t8];
t = t(1 : dimension);
C = Cartan(root_system, dimension);
W = Weyl(C);
N = size(W, 3);

Jacobian = sym(zeros(dimension));
for j2 = 1 : dimension
	e = zeros(dimension, 1);
	e(j2) = 1;
	for j1 = 1 : N
		Jacobian(:,j2)=Jacobian(:,j2)+i/N*W(:,:,j1)*e*exp(i*dot(W(:,:,j1)*e,t));
	end
end
Jacobian = simplify(Jacobian);
invJacobian = simplify(inv(Jacobian));
