function [IntT] = IntLookup_deltoid(N)
% create a table of Integration values over the deltoid for all multivariate Chebyshev polynomials
% within the dual fundamental domain for a given value of the downscaling lattice.
%
% Input:
%	N (1x1 double) = lattice downscaling factor
%
% Output:
%	IntT (max(k1) x max(k2) double) = integral over the deltoid of each Chebyshev polynomial in the fundamental domain
%		note that only such Chebyshev polynomials that appear get evaluated so matrix is sparse

% figure out which Chebyshev polynomials we need
g = dualfudo(N);

% construct the integral functions, or not since they aren't actually needed...
% k1~=0, k2~=0
% f1=inline('-1/k1*(1/(k2+2*k1)*(exp(i*(k2+2*k1)*2*pi/3)-1)-1/(k2+k1/2)*(exp(i*(k2+k1/2)*4*pi/3)-1)+1/(k2-k1)*exp(i*(k2-k1)*2*pi/3)*(exp(i*(k2-k1)*2*pi/3)-1))','k1','k2');
% k1==0, k2~=0
% f2=inline('2/3/k2^2*(-exp(i*k2*4*pi/3)+2*exp(i*k2*2*pi/3)-1)','k2');

% construct the Weyl group and Jacobian determinant modifiers to k
W=Weyl(Cartan('A',2));
l=[1 -1 1 -1 -2 2;1 -1 -2 2 1 -1;1 -1 1 -1 1 -1];

IntT=zeros(max(g(1,:))+1,max(g(2,:))+1);
for n=1:size(g,2)
  k=g(:,n);
  if k(1)~=k(2) & abs(k(1)-k(2))~=3
	continue; % integral over the deltoid is zero unless k1==k2 or abs(k(1)-k(2)) == 3
  end

  % construct the points to evaluate the integral at
  K=[];
  for j=1:6
	for j2=1:6
	  K=[K [W(:,:,j)*k+l(1:2,j2); l(3,j2)]];
	end
  end
  % sum over the integrals
  for j=1:36
	if K(1,j)==0 & K(2,j)==0
	  % IntT(g(1,n)+1,g(2,n)+1)=IntT(g(1,n)+1,g(2,n)+1)+K(3,j)*2*pi^2/3; % cancels due to symmetries
	  continue;
	end
	if K(1,j)==0
	  % IntT(g(1,n)+1,g(2,n)+1)=IntT(g(1,n)+1,g(2,n)+1)+K(3,j)*f2(K(2,j)); % cancels due to symmetries
	  continue;
	end
	if K(2,j)+K(1,j)/2==0
	  IntT(g(1,n)+1,g(2,n)+1)=IntT(g(1,n)+1,g(2,n)+1)+K(3,j)*4*pi*i/3/K(1,j);
	  continue;
	end
	if K(2,j)+2*K(1,j)==0
	  IntT(g(1,n)+1,g(2,n)+1)=IntT(g(1,n)+1,g(2,n)+1)+K(3,j)*-2*pi*i/3/K(1,j);
	  continue;
	end
	if K(2,j)-K(1,j)==0
	  IntT(g(1,n)+1,g(2,n)+1)=IntT(g(1,n)+1,g(2,n)+1)+K(3,j)*-2*pi*i/3/K(1,j);
	  continue;
	end
%     IntT(g(1,n)+1,g(2,n)+1)=IntT(g(1,n)+1,g(2,n)+1)+K(3,j)*f1(K(1,j),K(2,j)); % f1(k1,k2)=0 if |k1-k2|=0 (mod 3)
  end
end
% only need to remove imaginary round-off error if f1 and f2 are used
% i/2 from |J_x(z)|, -1/9 from |J_z(theta)|, 1/6 from 1/|W| and - from clockwise to anti-clockwise orientation of triangle in x and theta domains
IntT=IntT*i/9/6/2;
