function [I]=Interp(Points, Values, C)
% [I] = Interp(Points, Values, C)
% interpolate a function at several points, based on the function values on the lattice
%
% Input:
%	Points (2xn double) = desired interpolation points
%	Values (NxN double) = function values on the lattice
%	C (2x3 double) [optional] = corners of the triangle.
%		Defaults to [-1/3 -1/3 2/3;-1/sqrt(3) 1/sqrt(3) 0] if not specified.
%
% Output:
%	I (nx1 double) = interpolation values

% should probably check that the point is within the deltoid (or at least the triangle), otherwise the interpolation will be very bad!
if nargin < 3
  C=[-1/3 -1/3 2/3;-1/sqrt(3) 1/sqrt(3) 0];
end

N = size(Values,1); % get the downscaling
n = size(Points,2); % get the number of points to evaluate
if n==0
  I=[];
  return
end

a = fft2(Values)/N^2; % get the Fourier coefficients

[Tri,centroid] = ArbitraryTriangleToStandardTriangle(C);
Tri=inv(Tri);
z = Tri(1,:)*(Points-centroid(:,ones(n,1))) + i*Tri(2,:)*(Points-centroid(:,ones(n,1))); % obtain interpolation coordinates w.r.t. deltoid
T = T_A2(z.',N); % evaluate the Chebyshev polynomials at the interpolation points

g = dualfudo(N); % obtain symmetry information
Ndfudo = size(g,2);
G=Gdual;
Ndual = size(G,3);
[orbnum,orbact,iso] = orbit_info(G,g,N);
Niso = ones(n,1)*sum(iso~=0);

% perform the interpolation, discarding imaginary part which vanishes due to symmetries
I = real(sum(ones(n,1)*a(g(1,:)+1+N*g(2,:)).*T*Ndual./Niso,2));
