function [l]=Lagrange_basis_function(N,z,C)
% create a Lagrange basis function that is 1 at the point closest to z and zero elsewhere
%
% Input:
%	N (int) = downscaling factor
%	z (1x2 double optional) = the node nearest this point is used to construct the Lagrange basis function
%	C (2x3 double optional) = matrix containing triangle corners
%
% Output:
%	l (NxN double) = Lagrange basis function for the node closest to z

% figure out what was input
if nargin==2 && size(z,1)==2 && size(z,2)==3 % C was input instead of z
  C=z;
  clear z;
end
if exist('C')==0 % C wasn't input
  C=[-1/3 -1/3 2/3;-1/sqrt(3) 1/sqrt(3) 0];
end
[T,c]=ArbitraryTriangleToStandardTriangle(C);
if exist('z')==0 % z wasn't input
  th = linspace (0, pi, N/2+1); % actually (0, 2*pi, N+1), but duplicity removes half of them
  clf;
  hold on;
  for j = 0:N-1;
    [x, y] = deltocoord(th-pi*j/N, th+pi*j/N);
	X=T(1,:)*[x;y]+c(1);
	Y=T(2,:)*[x;y]+c(2);
    if rem(j,2)==0
      plot(X, Y, 'b-');
      plot(X, Y, 'b.', 'MarkerSize', 20);
    else
      plot(X, Y, 'b-');
    end
  end
  z=ginput(1);
end
z=(inv(T)*(z.'-c)).';
[ndx,z]=deltoIndex(N,z);
p=primalfudo(N);
W=Weyl(Cartan('A',2));
l=zeros(N);
for j=1:6;
  l(mod(W(:,1,j).'*p(:,ndx),N)+1,mod(W(:,2,j).'*p(:,ndx),N)+1)=1;
end;
