function Plot_Deltoid(f)
% Plot_Deltoid(f)
% Make surface plots of a function (or functions) on the deltoid in the {x_1,x_2} coordinates.
%
% Input:
%	f (N,N,d double) = d functions on the Deltoid

% Brett Ryland, November 2009.

N = size(f,1);
if size(f,2) ~= N
	error('Input has wrong dimensions');
end
d = size(f,3);

theta = [0:N-1]*2*pi/N;
x = (cos(theta.'*ones(1,N))+cos(ones(N,1)*theta)+cos(theta.'*ones(1,N)-ones(N,1)*theta))/3;
y = (sin(theta.'*ones(1,N))-sin(ones(N,1)*theta)-sin(theta.'*ones(1,N)-ones(N,1)*theta))/3;
for j=1:d
  figure
  surf(x,y,f(:,:,j));
  % shading interp;
  axis tight;
  zlim auto;
  title(sprintf('Function f(:,:,%d) plotted on the deltoid',j));
  xlabel('x');
  ylabel('y');
  rotate3d on;
  drawnow;
end
