function [I] = TriCCInt(f,C,N,IntT)
% [I] = TriCCInt(f,C,N,IntT)
% integral of a function defined on the triangle C
%
% Input:
%	f (string | NxN double) = function | function evaluated on deltoid points circumscribing the triangle C
%	C (2x3 double) = corners of the triangle
%	N (1x1 int) = downscaling factor
%	IntT (PxQ double) = table of integrals of Chebyshev polynomials from IntLookup_triangle(N)
%
% Output:
%	I (1x1 double) = integral of f over triangle

% Brett Ryland, Nov 2008

[T,centroid]=ArbitraryTriangleToStandardTriangle(C);
J=inv(T).';
theta = [0:N-1]*2*pi/N;
X = reshape(1/3*(exp(i*theta.'*ones(1,N)) + exp(-i*ones(N,1)*theta) + exp(-i*(theta.'*ones(1,N)-ones(N,1)*theta))),1,N*N);
Y=imag(X);
X=real(X);
x=reshape(T(1,:)*[X;Y],N,N)+centroid(1);
y=reshape(T(2,:)*[X;Y],N,N)+centroid(2);
if ischar(f) % f is a string
  f=eval(f);
elseif isnumeric(f) % f is a NxN double
  if size(f,1) ~= N | size(f,2) ~= N
	error('f is the wrong size')
  end
end

I=CCInt(f,N,IntT)*abs(det(T));
