function [D,x,y] = TriGrad(f,C,N)
% [D,x,y] = TriGrad(f,C,N)
% integral and gradient of a function defined on the triangle C
%
% Input:
%	f (string | NxN double) = function | function evaluated on deltoid points circumscribing the triangle C
%	C (2x3 double) = corners of the triangle
%	N (1x1 int) = downscaling factor
%
% Output:
%	D (NxNx2 double) = gradient at Chebyshev extremal points in the circumscribing deltoid
%	x (NxN double) = x coordinates of Chebyshev extremal points on triangle
%	y (NxN double) = y coordinates of Chebyshev extremal points on triangle

% Brett Ryland, Nov 2008

[T,centroid]=ArbitraryTriangleToStandardTriangle(C);
J=inv(T).';
theta = [0:N-1]*2*pi/N;
X = reshape(1/3*(exp(i*theta.'*ones(1,N)) + exp(-i*ones(N,1)*theta) + exp(-i*(theta.'*ones(1,N)-ones(N,1)*theta))),1,N*N);
Y=imag(X);
X=real(X);
x=reshape(T(1,:)*[X;Y],N,N)+centroid(1);
y=reshape(T(2,:)*[X;Y],N,N)+centroid(2);
if ischar(f) % f is a string
  f=eval(f);
elseif isnumeric(f) % f is a NxN double
  if size(f,1) ~= N | size(f,2) ~= N
	error('f is the wrong size')
  end
end

D=zeros(N,N,2);
d=Gradient(f,N);
D(:,:,1)=reshape(J(1,:)*[reshape(d(:,:,1),1,N*N);reshape(d(:,:,2),1,N*N)],N,N);
D(:,:,2)=reshape(J(2,:)*[reshape(d(:,:,1),1,N*N);reshape(d(:,:,2),1,N*N)],N,N);
