function [F]=TriangleMap(f,J)
% Construct the "surface area" integrand of a function on a nonlinear triangle mapped from the equilateral triangle.
%
% Input:
%	f (N,N) = function on the nonlinear triangle
%	J (2,2,N,N) or (2,2) = Jacobian of the nonlinear mapping at each point in the domain,
%		                   or a single Jacobian for a linear mapping.
%
% Output:
%	F (N,N) = integrand of the function on the equilateral triangle

N=size(f,1);
gradf=real(Gradient(f,N));
if length(size(J))==4
  for j1=1:N
	for j2=1:N
	  gradf(j1,j2,:)=inv(J(:,:,j1,j2)).'*reshape(gradf(j1,j2,:),2,1);
	end
  end
else
  %	phi is a linear mapping and J is a 2x2 matrix
  gradf=reshape(reshape(gradf,N*N,2)*inv(J),N,N,2);
end

% surface area function (adjust this for the desired integrand)
F=sqrt(1+gradf(:,:,1).^2+gradf(:,:,2).^2);

if length(size(J))==4
  for j1=1:N
	for j2=1:N
	  F(j1,j2)=F(j1,j2)*abs(det(J(:,:,j1,j2)));
	end
  end
else
  %	phi is a linear mapping and abs(det(J)) is a constant
  F=F*abs(det(J));
end
