function [W] = Weyl(C)
% [W] = Weyl(C)
% get all the matrices in the Weyl group (actually the transpose of the matrices),
% acting on vectors represented in LambdaH basis
%
% Input:
%	C (d*d double) = Cartan matrix.
% Output:
%	W (d*d*n double) = Weyl group for the given Cartan matrix, n is the size of the group.

% Brett Ryland, March 2010.

% construct the generators of the group
gen=MakeGen(C.');
% construct the rest of the group from the generators
W=MakeSym(gen);
% check if -I is in W. In that case, sort the matrices
for j = 1:size(W,3)
  if norm(W(:,:,j)+eye(size(C))) < 1e-7
	lenw = size(W,3)/2;
	for i = 1:lenw
	  for j = i+1:size(W,3)
		if norm(W(:,:,i)+W(:,:,j)) < 1e-7
		  W(:,:,j) = [];
		  j=j-1;
		end
	  end
	end
	W = cat(3,W,-W);
	break;
  end
end
% we actually want the transpose of the elements in the Weyl group
W=permute(W,[2 1 3]);
