function [dT_k] = dT_k_A2(theta, N)
% [dT_k] = dT_k_A2(theta, N)
% evaluate the derivatives of the Chebyshev polynomials on the fundamental domain of the A2 root system
%
% Input:
%	theta (2,1) = point at which to evaluate dT_k
%	N (int) = root lattice downscaling factor
%
% Output:
%	dT_k (N, N, 2) = evaluation for all dT_k = d(T_k) / d(theta) where k \in dualfudo.

% Brett Ryland, Nov 2007.

% get the fundamental domain
C = Cartan('A',2);
W = Weyl(C); % the group W.' = { g.' | g \in W }
normW = size(W,3);

% T_k = 1/|W| \sum_{g\in W} (g^Tk, theta)
% dT_k = 1/|W| \sum_{g\in W} i*[g^Tk] (g^Tk, theta)
% 	where (g^Tk, theta) = exp( i*dot(g^T * k, theta) )
% Note: the values of k are not [0:N-1] but rather [0:N/2-1 0 1-N/2:-1]
dT_k = zeros(N,N,2);
if rem(N,2) == 0
	K=[0:N/2-1 0 1-N/2:-1];
else
	K=[0:ceil(N/2)-1 1-ceil(N/2):-1];
end
for k1 = 1:N
	for k2 = 1:N
		k = [K(k1);K(k2)];
		for j = 1:normW
			dT_k(k1,k2,:) = dT_k(k1,k2,:) + reshape(W(:,:,j)*k*exp(i*dot(W(:,:,j)*k,theta)),1,1,2);
		end
	end
end
dT_k = dT_k*i/normW;
