function [ndx,z]=deltoIndex(N,z)
% return the index and coordinates of the point in the primalfudo nearest to z
% if z is not specified, then work interactively using ginput(1)
%
% Input:
%	N (integer) = downscaling factor
%	z (1x2, optional) = coordinates to find the point nearest
%
% Output:
%	ndx (integer) = index into the primalfudo of the closest point to z
%	z (1x2, double) = coordinates of the closest point to z
%
% Brett Ryland, November 2009

if nargin<2
  th = linspace (0, pi, N/2+1); % actually (0, 2*pi, N+1), but duplicity removes half of them
  clf;
  hold on;
  for j = 0:N-1;
	[x, y] = deltocoord(th-pi*j/N, th+pi*j/N);
	if rem(j,2)==0
	  plot(x, y, 'b-');
	  plot(x, y, 'b.', 'MarkerSize', 20);
	else
	  plot(x, y, 'b-');
	end
  end
  axis square;
  z=ginput(1);
end
p=primalfudo(N);
[x,y]=deltocoord(p(1,:)*2*pi/N,p(2,:)*2*pi/N);
[z,ndx]=min((x-z(1)).^2+(y-z(2)).^2);
[z(1),z(2)]=deltocoord(p(1,ndx)*2*pi/N,p(2,ndx)*2*pi/N);
