function [x,y] = deltocoord(th1,th2,N)
% DELTOCOORD - Coordinates on the deltoid
%
% function [x,y] = deltocoord(g1,g2,N)
%
% Coordinates on deltoid. 
%
% Output: x,y : points on deltoid
%
% Input: 
% Two arguments: [x,y] = deltocoord(th1,th2),
% where th1, th2 in [0,2pi] are angles.
% The mapping is 6--1, and its fundamental domain a triangle
% with corners in (0,0), (4*pi/3,2*pi/3), (2*pi/3,4*pi/3).
%
% Four arguments: [x,y] = deltocoord(g1,g2,N),
% where (g1, g2) \in ZN x ZN are gridpoints.
%
% deltocoord(g1,g2,N) == deltocoord(2*pi*g1/N,2*pi*g2/N)


% Hans Munthe-Kaas, 2005
if nargin==3
	th1 = th1*(2*pi/N);
	th2 = th2*(2*pi/N);
end
x = (cos(th1)+cos(th2)+cos(th1-th2))/3;
y = (sin(th1)-sin(th2)-sin(th1-th2))/3;
