function [Y,I,J] = uniq(X,tol)
% [Y,I,J] = uniq(X,tol)
% Obtain the sorted unique rows of the array X with tolerance tol.
%
% Input:
%	X - an array
%	tol - tolerance
%
% Output:
%	Y - the sorted unique rows of X
%	I - index vector such that Y = X(I,:)
%	J - index vector such that X = Y(J,:)
%
% Brett Ryland, August 2008

if size(X,1)<1
  Y=X;
  I=[];
  J=[];
  return
end
if nargin<2
  tol=eps;
end
if tol==0
  tol=eps;
end
D = round(1/tol*X)*tol;
[Z,I] = sortrows(D);
Z = X(I,:);
d = [true;any(abs(Z(1:end-1,:)-Z(2:end,:))>tol,2)];
Y = Z(d,:);
J = cumsum(d);
J(I) = J;
I = I(d);
